#import "DapiConnectPlugin.h"
#import "DPCClient.h"
#import "DapiConnect.h"

#define NULL_TO_NIL(obj) ({ __typeof__ (obj) __obj = (obj); [__obj isKindOfClass:[NSNull class]] ? nil : obj; })

@interface DapiConnectPlugin () <DPCConnectDelegate>

@property (nonatomic, copy) FlutterResult connectResult;
@property (nonatomic, copy) FlutterResult autoFlowResult;
@property (nonatomic, strong) NSMutableDictionary<NSString *, FlutterResult> *selectorResult;

@end

@implementation DapiConnectPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"dapi_connect"
            binaryMessenger:[registrar messenger]];
  DapiConnectPlugin* instance = [[DapiConnectPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (instancetype)init
{
    self = [super init];
    if (self) {
        self.selectorResult = [NSMutableDictionary dictionary];
    }
    return self;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    BOOL isImplemented = NO;
    SEL selector = NSSelectorFromString(call.method);
    if ([self respondsToSelector:selector]) {
        isImplemented = YES;
        
        [self.selectorResult setObject:result forKey:call.method];
        
        NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:[self methodSignatureForSelector:selector]];
        invocation.selector = selector;
        invocation.target = self;
        
        /*
         * Indices 0 and 1 indicate the hidden arguments self and _cmd,
         * respectively; you should set these values directly with the target and selector properties.
         * Use indices 2 and greater for the arguments normally passed in a message.
         */
        __block NSUInteger index = 2;
        NSArray<NSObject *> *arguments = call.arguments;
        [arguments enumerateObjectsUsingBlock:^(NSObject * _Nonnull arg, NSUInteger idx, BOOL * _Nonnull stop) {
            [invocation setArgument:&(arg) atIndex:index];
            index++;
        }];
        
        // Store result for a selector
        if (selector == @selector(presentConnect:)) {
            self.connectResult = result;
        }
        
        if (selector == @selector(presentAutoFlow:)) {
            self.autoFlowResult = result;
        }
        
        [invocation invoke];
        
        NSMethodSignature *signature = [invocation methodSignature];
        const char *type = [signature methodReturnType];
        
        if (strcmp(type, "v") != 0) {
            void *returnValue;
            [invocation getReturnValue:&returnValue];
            NSObject *resultSet = (__bridge NSObject *)returnValue;
            result(resultSet);
        }
    }
    
    if (!isImplemented) {
        result(FlutterMethodNotImplemented);
    }
}

- (void)initClient:(NSDictionary *)configurations {
    DPCConfigurations *configs = [self configurationsFromDictionary:configurations];
    __unused DPCClient *client = [[DPCClient alloc] initWithConfigurations:configs];
}

- (DPCClient *)clientWithConfigurations:(NSDictionary *)configurations {
    DPCConfigurations *configs = [self configurationsFromDictionary:configurations];
    DPCClient *client = [[DPCClient alloc] initWithConfigurations:configs];
    return client;
}

- (void)presentConnect:(NSDictionary *)configurations {
    DPCClient *client = [self clientWithConfigurations:configurations];
    client.connect.delegate = self;
    [client.connect present];
}

- (void)dismissConnect:(NSDictionary *)configurations {
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.connect dismissWithCompletion:nil];
}

- (void)presentAutoFlow:(NSDictionary *)configurations {
    DPCClient *client = [self clientWithConfigurations:configurations];
    client.autoFlow.connectDelegate = self;
    [client.autoFlow present];
}

- (void)dismissAutoFlow:(NSDictionary *)configurations {
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.autoFlow dismissWithCompletion:nil];
}

- (void)setAutoFlowMinimumAmount:(double)amount configurations:(NSDictionary *)configurations {
    DPCClient *client = [self clientWithConfigurations:configurations];
    client.autoFlow.minimumAmount = amount;
}

- (void)setAutoFlowMaximumAmount:(double)amount configurations:(NSDictionary *)configurations {
    DPCClient *client = [self clientWithConfigurations:configurations];
    client.autoFlow.maximumAmount = amount;
}

- (void)setUserID:(NSString *)userID configurations:(NSDictionary *)configurations {
    DPCClient *client = [self clientWithConfigurations:configurations];
    client.userID = NULL_TO_NIL(userID);
}

- (NSString *)getUserID:(NSDictionary *)configurations {
    DPCClient *client = [self clientWithConfigurations:configurations];
    return client.userID;
}

- (NSArray<NSDictionary<NSString *, id> *> *)dapiClientInstances {
    __block NSMutableArray<NSDictionary<NSString *, id> *> * dictionaryInstances = [NSMutableArray array];
    
    [DPCClient.instances enumerateObjectsUsingBlock:^(DPCClient * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        NSMutableDictionary<NSString *, id> *dictionaryInstance = [NSMutableDictionary dictionary];
        NSMutableDictionary<NSString *, id> *configurationsDictionary = [NSMutableDictionary dictionary];
        
        [configurationsDictionary setObject:obj.configurations.appKey forKey:@"appKey"];
        [configurationsDictionary setObject:obj.configurations.clientUserID forKey:@"clientUserID"];
        [configurationsDictionary setObject:obj.configurations.environment forKey:@"environment"];
        [configurationsDictionary setObject:obj.configurations.countries forKey:@"countries"];
        [configurationsDictionary setObject:[NSNumber numberWithBool:obj.configurations.autoTruncate] forKey:@"autoTruncate"];
        [configurationsDictionary setObject:[NSNumber numberWithBool:obj.configurations.isExperimental] forKey:@"isExperimental"];
        [configurationsDictionary setObject:obj.configurations.baseUrl.string forKey:@"baseURL"];

        if (obj.configurations.colorScheme) {
            [configurationsDictionary setObject:obj.configurations.colorScheme forKey:@"colorScheme"];
        }
        
        if (obj.configurations.endpoints) {
            [configurationsDictionary setObject:obj.configurations.endpoints forKey:@"endpoints"];
        }
        
        if (obj.configurations.endPointExtraQueryItems) {
            [configurationsDictionary setObject:obj.configurations.endPointExtraQueryItems forKey:@"endpointExtraQueryItems"];
        }
        
        if (obj.configurations.endPointExtraHeaderFields) {
            [configurationsDictionary setObject:obj.configurations.endPointExtraHeaderFields forKey:@"endpointExtraHeaderFields"];
        }
        
        if (obj.configurations.endPointExtraBody) {
            [configurationsDictionary setObject:obj.configurations.endPointExtraBody forKey:@"endpointExtraBody"];
        }
        
        [dictionaryInstance setObject:obj.userID ?: [NSNull null] forKey:@"userID"];
        
        [dictionaryInstance setObject:configurationsDictionary forKey:@"configurations"];
        
        [dictionaryInstances addObject:dictionaryInstance];
    }];
    
     return dictionaryInstances;
}

- (NSArray<NSDictionary<NSString *, id> *> *)dapiConnections:(NSDictionary *)configurations {
    DPCClient *client = [self clientWithConfigurations:configurations];
    
    NSMutableArray<NSDictionary<NSString *, id> *> *connections = [NSMutableArray array];
    
    [client.connect.getConnections enumerateObjectsUsingBlock:^(DPCConnectionDetails * _Nonnull connection, NSUInteger idx, BOOL * _Nonnull stop) {
        NSMutableDictionary<NSString *, id> *connectionDetails = [NSMutableDictionary dictionary];
        NSMutableArray<NSDictionary<NSString *, id> *> *accounts = [NSMutableArray array];
        
        [connection.accounts enumerateObjectsUsingBlock:^(DPCAccount * _Nonnull account, NSUInteger idx, BOOL * _Nonnull stop) {
            NSMutableDictionary<NSString *, id> *accountDetails = [NSMutableDictionary dictionary];
            accountDetails[@"iban"] = NULL_TO_NIL(account.iban);
            accountDetails[@"number"] = NULL_TO_NIL(account.number);
            accountDetails[@"type"] = NULL_TO_NIL(account.type);
            accountDetails[@"name"] = NULL_TO_NIL(account.name);
            accountDetails[@"id"] = NULL_TO_NIL(account.accountID);
            accountDetails[@"isFavourite"] = [NSNumber numberWithBool:account.isFavourite];
            
            NSMutableDictionary<NSString *, id> *currency = [NSMutableDictionary dictionary];
            currency[@"code"] = NULL_TO_NIL(account.currency.code);
            currency[@"name"] = NULL_TO_NIL(account.currency.name);
            
            accountDetails[@"currency"] = NULL_TO_NIL(currency);
            
            [accounts addObject:accountDetails];
        }];
        
        
        connectionDetails[@"userID"] = NULL_TO_NIL(connection.userID);
        connectionDetails[@"clientUserID"] = NULL_TO_NIL(connection.clientUserID);
        connectionDetails[@"bankID"] = NULL_TO_NIL(connection.bankID);
        connectionDetails[@"bankName"] = NULL_TO_NIL(connection.bankName);
        connectionDetails[@"countryName"] = NULL_TO_NIL(connection.countryName);
        connectionDetails[@"beneficiaryCoolDownPeriod"] = [NSNumber numberWithDouble:connection.beneficiaryCoolDownPeriod * 1000];
        connectionDetails[@"isCreateBeneficiaryEndpointRequired"] = [NSNumber numberWithBool:connection.isCreateBeneficiaryEndpointRequired];
        connectionDetails[@"accounts"] = accounts;;
        
        [connections addObject:connectionDetails];
    }];
    
    return connections;
}

- (void)delinkUser:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
    
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.auth delinkUser:^(DPCResult * _Nullable result, NSError * _Nullable error) {
        [self respondForDictionaryRepresentableObject:result error:error result:flutterResult];
    }];
}

- (void)getIdentity:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
    
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.data getIdentity:^(DPCIdentity * _Nullable identity, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:identity error:error result:flutterResult];
    }];
}

- (void)getAccounts:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
    
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.data getAccounts:^(NSArray<DPCAccount *> * _Nullable accounts, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:accounts error:error result:flutterResult];
    }];
}

- (void)getBalance:(NSString *)accountID configurations:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
    
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.data getBalanceForAccountID:accountID completion:^(DPCBalance * _Nullable balance, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:balance error:error result:flutterResult];
    }];
}

- (void)getTransactions:(NSString *)accountID fromTimestamp:(NSNumber *)fromTimestamp toTimestamp:(NSNumber *)toTimestamp configurations:(NSDictionary *)configurations {
    NSDate *fromDate = [NSDate dateWithTimeIntervalSince1970:fromTimestamp.doubleValue/1000];
    NSDate *toDate = [NSDate dateWithTimeIntervalSince1970:toTimestamp.doubleValue/1000];
    
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
    
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.data getTransactionsForAccountID:accountID fromDate:fromDate toDate:toDate completion:^(NSArray<DPCTransaction *> * _Nullable transactions, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:transactions error:error result:flutterResult];
    }];
}

- (void)getBeneficiaries:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
    
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.payment getBeneficiaries:^(NSArray<DPCBeneficiary *> * _Nullable beneficiaries, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:beneficiaries error:error result:flutterResult];
    }];
}

- (void)createBeneficiary:(NSDictionary *)beneficiaryInfo configurations:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
    
    DPCBeneficiaryInfo *info = [self nativeBeneficiaryInfoFromDictionary:beneficiaryInfo];
    if (!info) {
        NSError *constructionError = [NSError errorWithDomain:@"com.dapi.dapiconnect.flutter" code:1013 userInfo:@{NSLocalizedDescriptionKey: [NSString stringWithFormat:@"Couldn't construct native beneficiary info from Dictionary: %@", beneficiaryInfo]}];
        flutterResult([self flutterErrorFromNSError:constructionError]);
        return;
    }
    
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.payment createBeneficiaryWithInfo:info completion:^(DPCResult * _Nullable result, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:result error:error result:flutterResult];
    }];
}

- (void)createTransferWithReceiverID:(NSString *)receiverID senderID:(NSString *)senderID amount:(NSNumber *)amount configurations:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
        
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.payment createTransferWithSenderID:senderID amount:amount toReceiverID:receiverID completion:^(DPCResult * _Nullable result, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:result error:error result:flutterResult];
    }];
}

- (void)createTransferWithAccountNumber:(NSString *)accountNumber senderID:(NSString *)senderID amount:(NSNumber *)amount name:(NSString *)name configurations:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
        
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.payment createTransferWithSenderID:senderID amount:amount toAccountNumber:accountNumber name:name completion:^(DPCResult * _Nullable result, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:result error:error result:flutterResult];
    }];
}

- (void)createTransferWithIBAN:(NSString *)iban senderID:(NSString *)senderID amount:(NSNumber *)amount name:(NSString *)name configurations:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
        
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.payment createTransferWithSenderID:senderID amount:amount iban:iban name:name completion:^(DPCResult * _Nullable result, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:result error:error result:flutterResult];
    }];
}

- (void)getAccountMetadata:(NSDictionary *)configurations {
    NSString *resultKey = NSStringFromSelector(_cmd);
    FlutterResult flutterResult = [self.selectorResult objectForKey:resultKey];
    
    DPCClient *client = [self clientWithConfigurations:configurations];
    [client.metadata getAccountMetadata:^(DPCBankMetadata * _Nullable accounts, NSError * _Nullable error, NSString * _Nullable jobID) {
        [self respondForDictionaryRepresentableObject:accounts error:error result:flutterResult];
    }];
}

# pragma mark Helpers

// MARK: - Helper Methods
- (void)respondForDictionaryRepresentableObject:(NSObject *)object error:(NSError *)error result:(FlutterResult)result {
    
    if (error) {
        result([self flutterErrorFromNSError:error]);
    } if ([object isKindOfClass:[NSArray class]]) {
        NSMutableArray<NSDictionary *> *resultObjectsArray = [NSMutableArray array];
        NSArray *arrayObjects = (NSArray *)object;
        __block NSError *castingError;
        
        [arrayObjects enumerateObjectsUsingBlock:^(id  _Nonnull element, NSUInteger idx, BOOL * _Nonnull stop) {
            if ([element respondsToSelector:@selector(dictionaryRepresentation)]) {
                NSDictionary<NSString *, id> *dictionaryResponse = [element valueForKey:@"dictionaryRepresentation"];
                [resultObjectsArray addObject:dictionaryResponse];
            } else {
                castingError = [NSError errorWithDomain:@"com.dapi.dapiconnect.flutter" code:1012 userInfo:@{NSLocalizedDescriptionKey: @"Couldn't construct JSON representation of native array objects"}];
                *stop = YES;
            }
        }];
        if (castingError) {
            result([self flutterErrorFromNSError:castingError]);
        } else {
            result(resultObjectsArray);
        }
    } else {
        if ([object respondsToSelector:@selector(dictionaryRepresentation)]) {
            NSDictionary<NSString *, id> *dictionaryResponse = [object valueForKey:@"dictionaryRepresentation"];
            result(dictionaryResponse);
        } else {
            NSError *castingError = [NSError errorWithDomain:@"com.dapi.dapiconnect.flutter" code:1012 userInfo:@{NSLocalizedDescriptionKey: @"Couldn't construct JSON representation of native array objects"}];
            result([self flutterErrorFromNSError:castingError]);
        }
    }
}

- (FlutterError *)flutterErrorFromNSError:(NSError *)error {
    FlutterError *flutterError = [FlutterError errorWithCode:[NSString stringWithFormat:@"%@", [@(error.code) stringValue]] message:error.description details:nil];
    return flutterError;
}

- (DPCConfigurations *)configurationsFromDictionary:(NSDictionary *)dictionary {
    NSString *appKey = [dictionary objectForKey:@"appKey"];
    NSString *clientUserID = [dictionary objectForKey:@"clientUserID"];
    NSString *environment = [dictionary objectForKey:@"environment"];
    NSArray<NSString *> *countries = [dictionary objectForKey:@"countries"];
    BOOL autoTruncate = [NULL_TO_NIL([dictionary objectForKey:@"autoTruncate"]) boolValue];
    BOOL isExperimental = [NULL_TO_NIL([dictionary objectForKey:@"isExperimental"]) boolValue];
    NSString* baseURL = [dictionary objectForKey:@"baseURL"];
    NSString *colorScheme = [dictionary objectForKey:@"colorScheme"];
    NSDictionary* overridenEndpoints = [dictionary objectForKey:@"endpoints"];
    NSDictionary* endpointExtraQueryItems = [dictionary objectForKey:@"endpointExtraQueryItems"];
    NSDictionary* endpointExtraHeaderFields = [dictionary objectForKey:@"endpointExtraHeaderFields"];
    NSDictionary* endpointExtraBody = [dictionary objectForKey:@"endpointExtraBody"];
    
    // init native objects
    NSURLComponents *urlComponents = [[NSURLComponents alloc] initWithString:baseURL];
    DPCAppEnvironment parsedEnvironment = [self parseEnvironment:environment];
    
    DPCConfigurations *configurations = [[DPCConfigurations alloc] initWithAppKey:appKey baseUrl:urlComponents countries:countries clientUserID:clientUserID];
    configurations.colorScheme = [self parseColorScheme:colorScheme];
    configurations.environment = parsedEnvironment;
    configurations.isExperimental = isExperimental;
    configurations.autoTruncate = autoTruncate;
    configurations.endpoints = [self parseEndpoints:NULL_TO_NIL(overridenEndpoints)];
    configurations.endPointExtraBody = NULL_TO_NIL(endpointExtraBody);
    configurations.endPointExtraHeaderFields = NULL_TO_NIL(endpointExtraHeaderFields);
    configurations.endPointExtraQueryItems = NULL_TO_NIL(endpointExtraQueryItems);
    
    return configurations;
}

- (DPCAppEnvironment)parseEnvironment:(NSString *)environment {
    NSString *lowercasedEnv = [environment lowercaseString];
    if ([lowercasedEnv isEqualToString:@"sandbox"]) {
        return DPCAppEnvironmentSandbox;
    } else if ([lowercasedEnv isEqualToString:@"production"]) {
        return DPCAppEnvironmentProduction;
    } else {
        return DPCAppEnvironmentSandbox;
    }
}

- (DPCColorScheme)parseColorScheme:(NSString *)color {
    NSString *lowercasedColor = [color lowercaseString];
    if ([lowercasedColor isEqualToString:@"neon"]) {
        return DPCColorSchemeNeon;
    } else if ([lowercasedColor isEqualToString:@"bw"]) {
        return DPCColorSchemeBW;
    } else {
        return DPCColorSchemeGeneral;
    }
}

- (NSDictionary<DPCEndPoint, NSString *> *)parseEndpoints:(NSDictionary *)endpoints {
    NSMutableDictionary<DPCEndPoint, NSString *> *result = [NSMutableDictionary dictionary];
    
    [endpoints enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
        if ([key isKindOfClass:[NSString class]] && [obj isKindOfClass:[NSString class]]) {
            NSString *k = (NSString *)key;
            NSString *v = (NSString *)obj;
            
            if (![k isEqualToString:v]) {
                DPCEndPoint endpoint = [self nativeEndpointFromStringEndpoint:k];
                if (endpoint) {
                    [result setObject:v forKey:endpoint];
                }
            }
        }
    }];
    
    return result;
}

- (DPCEndPoint)nativeEndpointFromStringEndpoint:(NSString *)endpoint {
    if ([endpoint isEqualToString:@"exchangeToken"]) {
        return DPCEndPointExchangeToken;
    } else if ([endpoint isEqualToString:@"getIdentity"]) {
        return DPCEndPointGetIdentity;
    } else if ([endpoint isEqualToString:@"getAccounts"]) {
        return DPCEndPointGetAccounts;
    } else if ([endpoint isEqualToString:@"getAccountMetadata"]) {
        return DPCEndPointGetAccountMetadata;
    } else if ([endpoint isEqualToString:@"getBalance"]) {
        return DPCEndPointGetBalance;
    } else if ([endpoint isEqualToString:@"getTransactions"]) {
        return DPCEndPointGetTransactions;
    } else if ([endpoint isEqualToString:@"getBeneficiaries"]) {
        return DPCEndPointGetBeneficiaries;
    } else if ([endpoint isEqualToString:@"createBeneficiary"]) {
        return DPCEndPointCreateBeneficiary;
    } else if ([endpoint isEqualToString:@"createTransfer"]) {
        return DPCEndPointCreateTransfer;
    } else if ([endpoint isEqualToString:@"resumeJob"]) {
        return DPCEndPointResumeJob;
    } else {
        return nil;
    }
}

- (DPCBeneficiaryInfo *)nativeBeneficiaryInfoFromDictionary:(NSDictionary<NSString *,id> *)beneficiaryInfoDictionary {
    DPCBeneficiaryInfo *beneficiaryInfo = [[DPCBeneficiaryInfo alloc] init];
    NSDictionary<NSString *, id> *lineAddressDictionary = [beneficiaryInfoDictionary objectForKey:@"linesAddress"];
    beneficiaryInfo.linesAddress = [[DPCLinesAddress alloc] initWithDictionary:lineAddressDictionary];
    beneficiaryInfo.accountNumber = [beneficiaryInfoDictionary objectForKey:@"accountNumber"];
    beneficiaryInfo.name = [beneficiaryInfoDictionary objectForKey:@"name"];
    beneficiaryInfo.bankName = [beneficiaryInfoDictionary objectForKey:@"bankName"];
    beneficiaryInfo.swiftCode = [beneficiaryInfoDictionary objectForKey:@"swiftCode"];
    beneficiaryInfo.iban = [beneficiaryInfoDictionary objectForKey:@"iban"];
    beneficiaryInfo.phoneNumber = [beneficiaryInfoDictionary objectForKey:@"phoneNumber"];
    beneficiaryInfo.country = [beneficiaryInfoDictionary objectForKey:@"country"];
    beneficiaryInfo.branchAddress = [beneficiaryInfoDictionary objectForKey:@"branchAddress"];
    beneficiaryInfo.branchName = [beneficiaryInfoDictionary objectForKey:@"branchName"];
    return beneficiaryInfo;
}

- (void)connectBeneficiaryInfoForBankWithID:(nonnull NSString *)bankID beneficiaryInfo:(nonnull void (^)(DPCBeneficiaryInfo * _Nullable))info {
    info(nil);
//    - (void)sendMessage:(id _Nullable)message reply:(FlutterReply _Nullable)callback;
}

- (void)connectDidFailConnectingToBankID:(nonnull NSString *)bankID withError:(nonnull NSString *)error {
    NSDictionary *callbackDictionary = @{
        @"bankID": bankID ?: [NSNull null],
        @"error": error ?: [NSNull null],
    };
    if (self.connectResult) {
        self.connectResult(callbackDictionary);
        self.connectResult = nil;
    } else if (self.autoFlowResult) {
        self.autoFlowResult(callbackDictionary);
        self.autoFlowResult = nil;
    }
}

- (void)connectDidProceedWithBankID:(nonnull NSString *)bankID userID:(nonnull NSString *)userID {
}

- (void)connectDidSuccessfullyConnectToBankID:(nonnull NSString *)bankID userID:(nonnull NSString *)userID {
    NSDictionary *callbackDictionary = @{
        @"bankID": bankID,
        @"userID": userID,
    };
    if (self.connectResult) {
        self.connectResult(callbackDictionary);
        self.connectResult = nil;
    } else if (self.autoFlowResult) {
        self.autoFlowResult(callbackDictionary);
        self.autoFlowResult = nil;
    }
}

@end
