import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:intl/intl.dart';

class _Dapi {
  static const MethodChannel _channel = const MethodChannel('dapi_connect');

  static Future<Map<String, dynamic>> presentConnect(
      {@required DapiConfigurations configurations}) async {
    final List<dynamic> params = <dynamic>[configurations._toMap()];
    final Map result = await _channel.invokeMethod('presentConnect:', params);
    final Map<String, dynamic> typedResult = Map<String, dynamic>.from(result);
    return typedResult;
  }

  static void dismissConnect({@required DapiConfigurations configurations}) {
    final List<dynamic> params = <dynamic>[configurations._toMap()];
    _channel.invokeMethod('dismissConnect:', params);
  }

  static Future<List<Map<String, dynamic>>> dapiClientInstances() async {
    final List<dynamic> jsonInstances =
        await _channel.invokeMethod('dapiClientInstances');
    List<Map<String, dynamic>> mappedJsonInstances =
        jsonInstances.map((e) => Map<String, dynamic>.from(e)).toList();
    return mappedJsonInstances;
  }

  static Future<Map<String, dynamic>> presentAutoFlow(
      {@required DapiConfigurations configurations}) async {
    final List<dynamic> params = <dynamic>[configurations._toMap()];
    final Map result = await _channel.invokeMethod('presentAutoFlow:', params);
    final Map<String, dynamic> typedResult = Map<String, dynamic>.from(result);
    return typedResult;
  }

  static void dismissAutoFlow({@required DapiConfigurations configurations}) {
    final List<dynamic> params = <dynamic>[configurations._toMap()];
    _channel.invokeMethod('dismissAutoFlow:', params);
  }

  static void setAutoFlowMinimumAmount(
      {@required double amount, @required DapiConfigurations configurations}) {
    final List<dynamic> params = <dynamic>[amount, configurations._toMap()];
    _channel.invokeMethod('setAutoFlowMinimumAmount:configurations:', params);
  }

  static void setAutoFlowMaximumAmount(
      {@required double amount, @required DapiConfigurations configurations}) {
    final List<dynamic> params = <dynamic>[amount, configurations._toMap()];
    _channel.invokeMethod('setAutoFlowMaximumAmount:configurations:', params);
  }

  static void initClient({@required DapiConfigurations configurations}) {
    final List<dynamic> params = <dynamic>[configurations._toMap()];
    _channel.invokeMethod('initClient:', params);
  }

  static Future<List<Map<String, dynamic>>> dapiConnections(
      {@required DapiConfigurations configurations}) async {
    final List<dynamic> params = <dynamic>[configurations._toMap()];
    final List<dynamic> jsonInstances =
        await _channel.invokeMethod('dapiConnections:', params);
    List<Map<String, dynamic>> mappedJsonInstances =
        jsonInstances.map((e) => Map<String, dynamic>.from(e)).toList();
    return mappedJsonInstances;
  }

  static void setUserID(
      {@required String userID, @required DapiConfigurations configurations}) {
    final List<dynamic> params = [userID, configurations._toMap()];
    _channel.invokeMethod('setUserID:configurations:', params);
  }

  static Future<String> getUserID(
      {@required DapiConfigurations configurations}) async {
    final List<dynamic> params = [configurations._toMap()];
    final String userID = await _channel.invokeMethod('getUserID:', params);
    return userID;
  }

  static Future<DapiResult> delinkUser(
      {@required DapiConfigurations configurations}) async {
    final List<dynamic> params = [configurations._toMap()];
    final Map mapResult = await _channel.invokeMethod('delinkUser:', params);
    final Map<String, dynamic> typedMapResult =
        Map<String, dynamic>.from(mapResult);
    final DapiResult result = DapiResult._fromMap(typedMapResult);
    return result;
  }

  static Future<DapiIdentity> getIdentity(
      {@required DapiConfigurations configurations}) async {
    final List<dynamic> params = [configurations._toMap()];
    final Map mapResult = await _channel.invokeMethod('getIdentity:', params);
    final Map<String, dynamic> typedMapResult =
        Map<String, dynamic>.from(mapResult);
    final DapiIdentity identity = DapiIdentity._fromMap(typedMapResult);
    return identity;
  }

  static Future<List<DapiAccount>> getAccounts(
      {@required DapiConfigurations configurations}) async {
    final List<dynamic> params = [configurations._toMap()];
    final List<dynamic> jsonAccounts =
        await _channel.invokeMethod('getAccounts:', params);
    final List<Map<String, dynamic>> mappedJsonAccounts =
        jsonAccounts.map((e) => Map<String, dynamic>.from(e)).toList();
    final List<DapiAccount> typedAccounts =
        mappedJsonAccounts.map((e) => DapiAccount._fromMap(e)).toList();
    return typedAccounts;
  }

  static Future<DapiBalance> getBalance(
      {@required String accountID,
      @required DapiConfigurations configurations}) async {
    final List<dynamic> params = [accountID, configurations._toMap()];
    final Map mapResult =
        await _channel.invokeMethod('getBalance:configurations:', params);
    final Map<String, dynamic> typedMapResult =
        Map<String, dynamic>.from(mapResult);
    DapiBalance balance = DapiBalance._fromMap(typedMapResult);
    return balance;
  }

  static Future<List<DapiTransaction>> getTransactions(
      {@required String accountID,
      @required DateTime fromDate,
      @required DateTime toDate,
      @required DapiConfigurations configurations}) async {
    final List<dynamic> params = [
      accountID,
      fromDate.millisecondsSinceEpoch,
      toDate.millisecondsSinceEpoch,
      configurations._toMap()
    ];
    final List<dynamic> jsonTransactions = await _channel.invokeMethod(
        'getTransactions:fromTimestamp:toTimestamp:configurations:', params);
    final List<Map<String, dynamic>> mappedJsonTransactions =
        jsonTransactions.map((e) => Map<String, dynamic>.from(e)).toList();
    final List<DapiTransaction> typedTransactions =
        mappedJsonTransactions.map((e) => DapiTransaction._fromMap(e)).toList();
    return typedTransactions;
  }

  static Future<List<DapiBeneficiary>> getBeneficiaries(
      {@required DapiConfigurations configurations}) async {
    final List<dynamic> params = [configurations._toMap()];
    final List<dynamic> jsonBeneficiaries =
        await _channel.invokeMethod('getBeneficiaries:', params);
    final List<Map<String, dynamic>> mappedJsonBeneficiaries =
        jsonBeneficiaries.map((e) => Map<String, dynamic>.from(e)).toList();
    final List<DapiBeneficiary> typedBeneficiaries = mappedJsonBeneficiaries
        .map((e) => DapiBeneficiary._fromMap(e))
        .toList();
    return typedBeneficiaries;
  }

  static Future<DapiResult> createBeneficiary(
      {@required DapiBeneficiaryInfo beneficiaryInfo,
      @required DapiConfigurations configurations}) async {
    final List<dynamic> params = [
      beneficiaryInfo._toMap(),
      configurations._toMap()
    ];
    final Map mapResult = await _channel.invokeMethod(
        'createBeneficiary:configurations:', params);
    final Map<String, dynamic> typedMapResult =
        Map<String, dynamic>.from(mapResult);
    final DapiResult result = DapiResult._fromMap(typedMapResult);
    return result;
  }

  static Future<DapiResult> createTransferWithReceiverID(String receiverID,
      {@required String senderID,
      @required double amount,
      @required DapiConfigurations configurations}) async {
    final List<dynamic> params = [
      receiverID,
      senderID,
      amount,
      configurations._toMap()
    ];
    final Map mapResult =
        await _channel.invokeMethod('createTransferWithReceiverID:', params);
    final Map<String, dynamic> typedMapResult =
        Map<String, dynamic>.from(mapResult);
    final DapiResult result = DapiResult._fromMap(typedMapResult);
    return result;
  }

  static Future<DapiResult> createTransferWithAccountNumber(
      String accountNumber,
      {@required String senderID,
      @required double amount,
      @required String name,
      @required DapiConfigurations configurations}) async {
    final List<dynamic> params = [
      accountNumber,
      senderID,
      amount,
      name,
      configurations._toMap()
    ];
    final Map mapResult =
        await _channel.invokeMethod('createTransferWithAccountNumber:', params);
    final Map<String, dynamic> typedMapResult =
        Map<String, dynamic>.from(mapResult);
    final DapiResult result = DapiResult._fromMap(typedMapResult);
    return result;
  }

  static Future<DapiResult> createTransferWithIBAN(String iban,
      {@required String senderID,
      @required double amount,
      @required String name,
      @required DapiConfigurations configurations}) async {
    final List<dynamic> params = [
      iban,
      senderID,
      amount,
      name,
      configurations._toMap()
    ];
    final Map mapResult =
        await _channel.invokeMethod('createTransferWithIBAN:', params);
    final Map<String, dynamic> typedMapResult =
        Map<String, dynamic>.from(mapResult);
    final DapiResult result = DapiResult._fromMap(typedMapResult);
    return result;
  }

  static Future<DapiMetadata> getAccountMetadata({@required DapiConfigurations configurations}) async {
    final List<dynamic> params = [configurations._toMap()];
    final Map mapResult = await _channel.invokeMethod('getAccountMetadata:', params);
    final Map<String, dynamic> typedMapResult = Map<String, dynamic>.from(mapResult);
    final DapiMetadata metadata = DapiMetadata._fromMap(typedMapResult);
    return metadata;

  }
}

enum DapiEnvironment { production, sandbox }

enum DapiEndpoint {
  exchangeToken,
  getIdentity,
  getAccounts,
  getAccountMetadata,
  getBalance,
  getTransactions,
  getBeneficiaries,
  createBeneficiary,
  createTransfer,
  resumeJob,
}

class DapiConfigurations {
  final String appKey;
  final String baseURL;
  final List<String> countries;
  final String clientUserID;
  DapiEnvironment environment;
  bool isExperimental;
  bool autoTruncate;
  Map<DapiEndpoint, List<Map<String, String>>> endpointExtraQueryItems;
  Map<DapiEndpoint, Map<String, String>> endpointExtraHeaderFields;
  // Dynamic is for primitive types only
  Map<DapiEndpoint, Map<String, dynamic>> endpointExtraBody;
  Map<DapiEndpoint, String> endpoints;

  DapiConfigurations(
      this.appKey, this.baseURL, this.countries, this.clientUserID,
      {this.environment = DapiEnvironment.sandbox,
      this.isExperimental = false,
      this.autoTruncate = false});

  Map<String, dynamic> _toMap() {
    Map<String, List<Map<String, String>>> stringifiedEndpointExtraQueryItems;
    Map<String, Map<String, String>> stringifiedEndpointExtraHeaderFields;
    Map<String, Map<String, dynamic>> stringifiedEndpointExtraBody;
    Map<String, String> stringifiedEndpoints;

    if (endpointExtraQueryItems != null) {
      stringifiedEndpointExtraQueryItems = endpointExtraQueryItems.map((key, value) => MapEntry(key.toString().split('.').last, value));
    }
    if (endpointExtraHeaderFields != null) {
      stringifiedEndpointExtraHeaderFields = endpointExtraHeaderFields.map((key, value) => MapEntry(key.toString().split('.').last, value));
    }
    if (endpointExtraBody != null) {
      stringifiedEndpointExtraBody = endpointExtraBody.map((key, value) => MapEntry(key.toString().split('.').last, value));
    }
    if (endpoints != null) {
      stringifiedEndpoints = endpoints.map((key, value) => MapEntry(key.toString().split('.').last, value));
    }

    return <String, dynamic>{
      "appKey": appKey,
      "baseURL": baseURL,
      "countries": countries,
      "clientUserID": clientUserID,
      "environment": environment.toString().split(".").last,
      "isExperimental": isExperimental,
      "autoTruncate": autoTruncate,
      "endpointExtraQueryItems": stringifiedEndpointExtraQueryItems,
      "endpointExtraHeaderFields": stringifiedEndpointExtraHeaderFields,
      "endpointExtraBody": stringifiedEndpointExtraBody,
      "endpoints": stringifiedEndpoints,
    };
  }

  static DapiConfigurations _fromMap(Map<String, dynamic> map) {
    return DapiConfigurations(map['appKey'], map['baseURL'],
        List<String>.from(map['countries']), map['clientUserID'],
        environment: DapiEnvironment.sandbox,
        isExperimental: map['isExperimental'],
        autoTruncate: map['autoTruncate']);
  }
}

class DapiClient {
  final DapiConfigurations configurations;
  _DapiConnect _connect;
  _AutoFlow _autoFlow;
  _DapiAuth _auth;
  _DapiData _data;
  _DapiPayment _payment;
  _DapiMetadata _metadata;

  DapiClient(this.configurations) {
    _connect = _DapiConnect(this);
    _autoFlow = _AutoFlow(this);
    _auth = _DapiAuth(this);
    _data = _DapiData(this);
    _payment = _DapiPayment(this);
    _metadata = _DapiMetadata(this);
    _Dapi.initClient(configurations: configurations);
  }

  _DapiConnect get connect => _connect;
  _AutoFlow get autoFlow => _autoFlow;
  _DapiAuth get auth => _auth;
  _DapiData get data => _data;
  _DapiPayment get payment => _payment;
  _DapiMetadata get metadata => _metadata;

  set userID(String userID) {
    _Dapi.setUserID(userID: userID, configurations: configurations);
  }

  Future<String> getUserID() async {
    return await _Dapi.getUserID(configurations: configurations);
  }

  static Future<List<DapiClient>> get instances async {
    final List<Map<String, dynamic>> jsonClientInstances =
        await _Dapi.dapiClientInstances();
    List<DapiClient> clientInstances = [];
    jsonClientInstances.forEach((element) {
      String userID = element['userID'];
      Map<String, dynamic> configurationsMap =
          Map<String, dynamic>.from(element['configurations']);

      DapiConfigurations configurations =
          DapiConfigurations._fromMap(configurationsMap);
      DapiClient client = DapiClient(configurations);
      if (userID != null) {
        client.userID = userID;
      }
      clientInstances.add(client);
    });
    return clientInstances;
  }
}

class _DapiData {
  final DapiClient _client;

  _DapiData(this._client);

  Future<DapiIdentity> getIdentity() async {
    DapiIdentity result =
        await _Dapi.getIdentity(configurations: _client.configurations);
    return result;
  }

  Future<List<DapiAccount>> getAccounts() async {
    List<DapiAccount> accounts =
        await _Dapi.getAccounts(configurations: _client.configurations);
    return accounts;
  }

  Future<DapiBalance> getBalance({@required String accountID}) async {
    DapiBalance balance = await _Dapi.getBalance(
        accountID: accountID, configurations: _client.configurations);
    return balance;
  }

  Future<List<DapiTransaction>> getTransactions(
      {@required String accountID,
      @required DateTime fromDate,
      @required DateTime toDate}) async {
    List<DapiTransaction> transactions = await _Dapi.getTransactions(
        accountID: accountID,
        fromDate: fromDate,
        toDate: toDate,
        configurations: _client.configurations);
    return transactions;
  }
}

class _DapiAuth {
  final DapiClient _client;

  _DapiAuth(this._client);

  Future<DapiResult> delinkUser() async {
    DapiResult result =
        await _Dapi.delinkUser(configurations: _client.configurations);
    return result;
  }
}

class _DapiPayment {
  final DapiClient _client;

  _DapiPayment(this._client);

  Future<List<DapiBeneficiary>> getBeneficiaries() async {
    List<DapiBeneficiary> beneficiaries =
        await _Dapi.getBeneficiaries(configurations: _client.configurations);
    return beneficiaries;
  }

  Future<DapiResult> createBeneficiary(
      {@required DapiBeneficiaryInfo beneficiaryInfo}) async {
    DapiResult result = await _Dapi.createBeneficiary(
        beneficiaryInfo: beneficiaryInfo,
        configurations: _client.configurations);
    return result;
  }

  Future<DapiResult> createTransferWithReceiverID(String receiverID,
      {@required String senderID, @required double amount}) async {
    DapiResult result = await _Dapi.createTransferWithReceiverID(receiverID,
        senderID: senderID,
        amount: amount,
        configurations: _client.configurations);
    return result;
  }

  Future<DapiResult> createTransferWithAccountNumber(String accountNumber,
      {@required String senderID,
      @required double amount,
      @required String name}) async {
    DapiResult result = await _Dapi.createTransferWithAccountNumber(
        accountNumber,
        senderID: senderID,
        amount: amount,
        name: name,
        configurations: _client.configurations);
    return result;
  }

  Future<DapiResult> createTransferWithIBAN(String iban,
      {@required String senderID,
      @required double amount,
      @required String name}) async {
    DapiResult result = await _Dapi.createTransferWithIBAN(iban,
        senderID: senderID,
        amount: amount,
        name: name,
        configurations: _client.configurations);
    return result;
  }
}

class _DapiMetadata {
  final DapiClient _client;

  _DapiMetadata(this._client);

  Future<DapiMetadata> getAccountMetadata() async {
    final DapiMetadata metadata = await _Dapi.getAccountMetadata(configurations: _client.configurations);
    return metadata;
  }

}

class DapiMetadata {
  final DapiLineAddress address;
  final String bankName;
  final String branchAddress;
  final String branchName;
  final String swiftCode;
  final bool isCreateBeneficiaryEndpointRequired;
  final List<DapiTransferBound> transferBounds;
  final DapiPair country;
  final DapiMeasure transactionRange;
  final DapiMeasure beneficiaryCoolDownPeriod;
  final DapiValidator validator;

  DapiMetadata(
      {@required this.address,
      @required this.bankName,
      @required this.branchAddress,
      @required this.branchName,
      @required this.swiftCode,
      @required this.isCreateBeneficiaryEndpointRequired,
      @required this.transferBounds,
      @required this.country,
      @required this.transactionRange,
      @required this.beneficiaryCoolDownPeriod,
      @required this.validator});

  static DapiMetadata _fromMap(Map<String, dynamic> map) {
    Map<String, dynamic> addressMap = Map<String, dynamic>.from(map['address']);

    List<dynamic> transferBoundsList = List.from(map['transferBounds']).toList();
    List<Map<String, dynamic>> transferBoundsTypedMappedList = transferBoundsList.map((e) => Map<String, dynamic>.from(e)).toList();
    List<DapiTransferBound> typedTransferBounds = transferBoundsTypedMappedList.map((e) => DapiTransferBound._fromMap(e)).toList();
    
    Map<dynamic, dynamic> validatorsMap = map['validators'];
    DapiValidator validators;
    if (validatorsMap != null) {
      Map<String, dynamic> validatorsTypedMap = Map<String, dynamic>.from(validatorsMap);
      validators = DapiValidator._fromMap(Map<String, dynamic>.from(validatorsTypedMap));
    }

    return DapiMetadata(
      address: DapiLineAddress._fromMap(addressMap),
      bankName: map['bankName'],
      branchAddress: map['branchAddress'],
      branchName: map['branchName'],
      swiftCode: map['swiftCode'],
      isCreateBeneficiaryEndpointRequired: map['isCreateBeneficiaryEndpointRequired'],
      transferBounds: typedTransferBounds,
      country: DapiPair._fromMap(Map<String, dynamic>.from(map['country'])),
      transactionRange: DapiMeasure._fromMap(Map<String, dynamic>.from(map['transactionRange'])),
      beneficiaryCoolDownPeriod: DapiMeasure._fromMap(Map<String, dynamic>.from(map['beneficiaryCoolDownPeriod'])),
      validator: validators,
    );
    
  }
}

class DapiValidator {
  final DapiCreateBeneficiaryValidator typeSameCreateBeneficiaryValidator;
  final DapiCreateBeneficiaryValidator typeLocalCreateBeneficiaryValidator;

  DapiValidator(
      {@required this.typeSameCreateBeneficiaryValidator,
      @required this.typeLocalCreateBeneficiaryValidator});

  static DapiValidator _fromMap(Map<String, dynamic> map) {
    Map<String, dynamic> createBeneficiaryValidatorMap =
        Map<String, dynamic>.from(map['createBeneficiary']);
    Map<String, dynamic> same =
        Map<String, dynamic>.from(createBeneficiaryValidatorMap['same']);
    Map<String, dynamic> local =
        Map<String, dynamic>.from(createBeneficiaryValidatorMap['local']);

    return DapiValidator(
        typeSameCreateBeneficiaryValidator:
            DapiCreateBeneficiaryValidator._fromMap(same),
        typeLocalCreateBeneficiaryValidator:
            DapiCreateBeneficiaryValidator._fromMap(local));
  }
}

class DapiCreateBeneficiaryValidator {
  final DapiValidatorPropertyRequrement name;
  final DapiValidatorPropertyRequrement nickname;
  final DapiValidatorPropertyRequrement swiftCode;
  final DapiValidatorPropertyRequrement branchAddress;
  final DapiValidatorPropertyRequrement branchName;
  final DapiValidatorPropertyRequrement country;
  final DapiValidatorPropertyRequrement phoneNumber;
  final DapiValidatorPropertyRequrement sortCode;
  final DapiValidatorPropertyRequrement addressLine1;
  final DapiValidatorPropertyRequrement addressLine2;
  final DapiValidatorPropertyRequrement addressLine3;

  DapiCreateBeneficiaryValidator(
      {@required this.name,
      @required this.nickname,
      @required this.swiftCode,
      @required this.branchAddress,
      @required this.branchName,
      @required this.country,
      @required this.phoneNumber,
      @required this.sortCode,
      @required this.addressLine1,
      @required this.addressLine2,
      @required this.addressLine3});

  static DapiCreateBeneficiaryValidator _fromMap(Map<String, dynamic> map) {
    final Map<String, dynamic> addressMap = Map<String, dynamic>.from(map['address']);
    
    final nameObject =  map['name'];
    final nicknameObject =  map['nickname'];
    final swiftCodeObject =  map['swiftCode'];
    final branchAddressObject =  map['branchAddress'];
    final branchNameObject =  map['branchName'];
    final countryObject =  map['country'];
    final phoneNumberObject =  map['phoneNumber'];
    final sortCodeObject =  map['sortCode'];
    
    final line1Object =  addressMap['line1'];
    final line2Object =  addressMap['line2'];
    final line3Object =  addressMap['line3'];

    
    return DapiCreateBeneficiaryValidator(
      name: nameObject == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(nameObject)),
      nickname: nicknameObject == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(nicknameObject)),
      swiftCode: swiftCodeObject == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(swiftCodeObject)),
      branchAddress: branchAddressObject == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(branchAddressObject)),
      branchName: branchNameObject == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(branchNameObject)),
      country: countryObject == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(countryObject)),
      phoneNumber: phoneNumberObject == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(phoneNumberObject)),
      sortCode: sortCodeObject == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(sortCodeObject)),
      addressLine1: line1Object == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(line1Object)),
      addressLine2: line2Object == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(line2Object)),
      addressLine3: line3Object == null ? null : DapiValidatorPropertyRequrement._fromMap(Map<String, dynamic>.from(line3Object)),
    );
  }
}

class DapiValidatorPropertyRequrement {
  final bool isRequired;
  final bool isOptional;
  final int length;
  final String allowedCharacters;

  DapiValidatorPropertyRequrement(
      {@required this.isRequired,
      @required this.isOptional,
      @required this.length,
      @required this.allowedCharacters});

  static DapiValidatorPropertyRequrement _fromMap(Map<String, dynamic> map) {
    return DapiValidatorPropertyRequrement(
      isRequired: map['isRequired'],
      isOptional: map['isOptional'],
      length: map['length'],
      allowedCharacters: map['allowedCharacters'],
    );
  }
}

class DapiTransferBound {
  final num minimum;
  final DapiPair currency;
  final String type;

  DapiTransferBound({@required this.minimum, @required this.currency, @required this.type});

  static DapiTransferBound _fromMap(Map<String, dynamic> map) {
    final Map<String, dynamic> currency = Map<String, dynamic>.from(map['currency']);
    return DapiTransferBound(minimum: map['minimum'], currency: DapiPair._fromMap(currency), type: map['type']);
  }
}

class _DapiConnect {
  final DapiClient _client;

  _DapiConnect(this._client);

  Future<Map<String, dynamic>> present() async {
    Map<String, dynamic> result =
        await _Dapi.presentConnect(configurations: _client.configurations);
    return result;
  }

  void dismiss() {
    _Dapi.dismissConnect(configurations: _client.configurations);
  }

  Future<List<DapiConnection>> getConnections() async {
    final List<Map<String, dynamic>> jsonConnections =
        await _Dapi.dapiConnections(configurations: _client.configurations);
    List<DapiConnection> connections =
        jsonConnections.map((e) => DapiConnection._fromMap(e)).toList();
    return connections;
  }
}

class _AutoFlow {
  final DapiClient _client;
  double _minimumAmount;
  double _maximumAmount;

  double get minimumAmount {
    return _minimumAmount;
  }

  set minimumAmount(double amount) {
    _minimumAmount = amount;
    _Dapi.setAutoFlowMinimumAmount(
        amount: amount, configurations: _client.configurations);
  }

  double get maximumAmount {
    return _maximumAmount;
  }

  set maximumAmount(double amount) {
    _maximumAmount = amount;
    _Dapi.setAutoFlowMaximumAmount(
        amount: amount, configurations: _client.configurations);
  }

  _AutoFlow(this._client);

  Future<Map<String, dynamic>> present() async {
    Map<String, dynamic> result =
        await _Dapi.presentAutoFlow(configurations: _client.configurations);
    return result;
  }

  void dismiss() {
    _Dapi.dismissAutoFlow(configurations: _client.configurations);
  }
}

class DapiConnection {
  final String userID;
  final String clientUserID;
  final String bankID;
  final String bankName;
  final String countryName;
  final double beneficiaryCoolDownPeriod;
  final bool isCreateBeneficiaryEndpointRequired;
  final List<DapiAccount> accounts;

  DapiConnection(
      {@required this.userID,
      @required this.clientUserID,
      @required this.bankID,
      @required this.bankName,
      @required this.countryName,
      @required this.beneficiaryCoolDownPeriod,
      @required this.isCreateBeneficiaryEndpointRequired,
      @required this.accounts});

  static DapiConnection _fromMap(Map<String, dynamic> map) {
    List<dynamic> jsonAccounts = List<dynamic>.from(map['accounts']);
    List<Map<String, dynamic>> mappedAccounts =
        jsonAccounts.map((e) => Map<String, dynamic>.from(e)).toList();
    List<DapiAccount> accounts =
        mappedAccounts.map((e) => DapiAccount._fromMap(e)).toList();

    return DapiConnection(
        userID: map['userID'],
        clientUserID: map['clientUserID'],
        bankID: map['bankID'],
        bankName: map['bankName'],
        countryName: map['countryName'],
        beneficiaryCoolDownPeriod: map['beneficiaryCoolDownPeriod'],
        isCreateBeneficiaryEndpointRequired:
            map['isCreateBeneficiaryEndpointRequired'],
        accounts: accounts);
  }
}

class DapiAccount {
  final String iban;
  final String number;
  final DapiPair currency;
  final String type;
  final String name;
  final String id;
  final bool isFavourite;

  DapiAccount(
      {@required this.iban,
      @required this.number,
      @required this.currency,
      @required this.type,
      @required this.name,
      @required this.id,
      @required this.isFavourite});

  static DapiAccount _fromMap(Map<String, dynamic> map) {
    Map<String, dynamic> currencyMap =
        Map<String, dynamic>.from(map['currency']);
    return DapiAccount(
        iban: map['iban'],
        number: map['number'],
        currency: DapiPair._fromMap(currencyMap),
        type: map['type'],
        name: map['name'],
        id: map['id'],
        isFavourite: map['isFavourite']);
  }
}

class DapiPair {
  final String code;
  final String name;

  DapiPair({@required this.code, @required this.name});

  static DapiPair _fromMap(Map<String, dynamic> map) {
    return DapiPair(code: map['code'], name: map['name']);
  }
}

class DapiMeasure {
  final String unit;
  final num value;

  DapiMeasure({@required this.unit, @required this.value});

  static DapiMeasure _fromMap(Map<String, dynamic> map) {
    return DapiMeasure(unit: map['unit'], value: map['value']);
  }
}

class DapiResult {
  final String jobID;
  final String status;
  final String message;
  final bool success;

  DapiResult(
      {@required this.jobID,
      @required this.status,
      @required this.message,
      @required this.success});

  static DapiResult _fromMap(Map<String, dynamic> map) {
    return DapiResult(
        jobID: map['jobID'],
        status: map['status'],
        message: map['message'],
        success: map['success']);
  }
}

class DapiNumber {
  final String type;
  final String value;

  DapiNumber({@required this.type, @required this.value});

  static DapiNumber _fromMap(Map<String, dynamic> map) {
    return DapiNumber(type: map['type'], value: map['value']);
  }
}

class DapiIdentification {
  final String type;
  final String value;

  DapiIdentification({@required this.type, @required this.value});

  static DapiIdentification _fromMap(Map<String, dynamic> map) {
    return DapiIdentification(type: map['type'], value: map['value']);
  }
}

class DapiAddress {
  final String flat;
  final String building;
  final String area;
  final String poBox;
  final String city;
  final String state;
  final String country;
  final String full;

  DapiAddress(
      {@required this.flat,
      @required this.building,
      @required this.area,
      @required this.poBox,
      @required this.city,
      @required this.state,
      @required this.country,
      @required this.full});

  static DapiAddress _fromMap(Map<String, dynamic> map) {
    return DapiAddress(
      flat: map['flat'],
      building: map['building'],
      area: map['area'],
      poBox: map['poBox'],
      city: map['city'],
      state: map['state'],
      country: map['country'],
      full: map['full'],
    );
  }
}

class DapiIdentity {
  final String nationality;
  final String dateOfBirth;
  final List<DapiNumber> numbers;
  final String emailAddress;
  final String name;
  final DapiAddress address;
  final List<DapiIdentification> identifications;

  DapiIdentity(
      {@required this.nationality,
      @required this.dateOfBirth,
      @required this.numbers,
      @required this.emailAddress,
      @required this.name,
      @required this.address,
      @required this.identifications});

  static DapiIdentity _fromMap(Map<String, dynamic> map) {
    List<dynamic> numbersMap = map['numbers'];
    Map<String, dynamic> addressMap = Map<String, dynamic>.from(map['address']);
    List<dynamic> identificationsMap = map['identifications'];

    List<DapiNumber> numbers;
    DapiAddress address;
    List<DapiIdentification> identifications;

    if (numbersMap != null) {
      final List<Map<String, dynamic>> numbersList =
          List<Map<String, dynamic>>.from(numbersMap);
      numbers = numbersList.map((e) => DapiNumber._fromMap(e)).toList();
    }

    if (addressMap != null) {
      address = DapiAddress._fromMap(addressMap);
    }

    if (identificationsMap != null) {
      final List<Map<String, dynamic>> identificationsList =
          List<Map<String, dynamic>>.from(identificationsMap);
      identifications = identificationsList
          .map((e) => DapiIdentification._fromMap(e))
          .toList();
    }

    return DapiIdentity(
        nationality: map['nationality'],
        dateOfBirth: map['dateOfBirth'],
        numbers: numbers,
        emailAddress: map['emailAddress'],
        name: map['name'],
        address: address,
        identifications: identifications);
  }
}

class DapiBalance {
  final double amount;
  final DapiPair currency;
  final String accountNumber;

  DapiBalance(
      {@required this.amount,
      @required this.currency,
      @required this.accountNumber});

  static DapiBalance _fromMap(Map<String, dynamic> map) {
    Map<String, dynamic> currencyMap =
        Map<String, dynamic>.from(map['currency']);
    return DapiBalance(
        amount: map['amount'],
        currency: DapiPair._fromMap(currencyMap),
        accountNumber: map['accountNumber']);
  }
}

class DapiTransaction {
  final double beforeAmount;
  final double afterAmount;
  final double amount;
  final DapiPair currency;
  final DateTime date;
  final String description;
  final String details;
  final String type;

  DapiTransaction(
      {@required this.beforeAmount,
      @required this.afterAmount,
      @required this.amount,
      @required this.currency,
      @required this.date,
      @required this.description,
      @required this.details,
      @required this.type});

  static DapiTransaction _fromMap(Map<String, dynamic> map) {
    Map<String, dynamic> currencyMap =
        Map<String, dynamic>.from(map['currency']);
    DapiPair currency = DapiPair._fromMap(currencyMap);

    String stringDate = map['date'];
    DateTime date = DateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ").parse(stringDate);

    return DapiTransaction(
      beforeAmount: map['beforeAmount'].toDouble(),
      afterAmount: map['afterAmount'].toDouble(),
      amount: map['amount'].toDouble(),
      currency: currency,
      date: date,
      description: map['description'],
      details: map['details'],
      type: map['type'],
    );
  }
}

class DapiBeneficiary {
  final String name;
  final String accountNumber;
  final String iban;
  final String type;
  final String status;
  final String id;

  DapiBeneficiary(
      {@required this.name,
      @required this.accountNumber,
      @required this.iban,
      @required this.type,
      @required this.status,
      @required this.id});

  static DapiBeneficiary _fromMap(Map<String, dynamic> map) {
    return DapiBeneficiary(
      name: map['name'],
      accountNumber: map['accountNumber'],
      iban: map['iban'],
      type: map['type'],
      status: map['status'],
      id: map['id'],
    );
  }
}

class DapiLineAddress {
  final String line1;
  final String line2;
  final String line3;

  DapiLineAddress(
      {@required this.line1, @required this.line2, @required this.line3});
      
  Map<String, dynamic> _toMap() {
    return {
      "line1": line1,
      "line2": line2,
      "line3": line3,
    };
  }

  static DapiLineAddress _fromMap(Map<String, dynamic> map) {
    return DapiLineAddress(line1: map['line1'], line2: map['line2'], line3: map['line3']);
  }
}

class DapiBeneficiaryInfo {
  final DapiLineAddress linesAddress;
  final String accountNumber;
  final String name;
  final String bankName;
  final String swiftCode;
  final String iban;
  final String phoneNumber;
  final String country;
  final String branchAddress;
  final String branchName;

  DapiBeneficiaryInfo(
      {@required this.linesAddress,
      @required this.accountNumber,
      @required this.name,
      @required this.bankName,
      @required this.swiftCode,
      @required this.iban,
      @required this.phoneNumber,
      @required this.country,
      @required this.branchAddress,
      @required this.branchName});

  Map<String, dynamic> _toMap() {
    return {
      "linesAddress": linesAddress._toMap(),
      "accountNumber": accountNumber,
      "name": name,
      "bankName": bankName,
      "swiftCode": swiftCode,
      "iban": iban,
      "phoneNumber": phoneNumber,
      "country": country,
      "branchAddress": branchAddress,
      "branchName": branchName,
    };
  }
}
